"""
    register.py - this module is a part of clb (cloud load balancer)

    Copyright 2010, Canonical Ltd.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

from twisted.web.resource import Resource, ErrorPage
import os
import sqlite3

serverdb = "clb.db"
pre_shared_key = "xYztest1"

class RegisterResource(Resource):

    def getChild(self, name, request):
        # verify auth
        try:
            h = request.getAllHeaders()['X-CloudLoadBalancer-Auth']
            if h != pre_shared_key:
                return ErrorPage(403,'Auth Required', '')
        except KeyError:
            return ErrorPage(403,'Auth Required', '')
        if name == 'addserver':
            return AddServerResource()
        return self

    def render_GET(self, request):
        return request.URLPath()


class AddServerResource(RegisterResource):

    def getChild(self, name, request):
        """ http://$who/addserver/+service/$what/$ip/$where """
        self.messages='--'
        parts = request.URLPath().pathList()
        self.dparts = dict()
        storepart = None
        for part in parts:
            if storepart and len(storepart):
                self.dparts[storepart]=part
                storepart = None
                continue
            if len(part) and part[0:3] == '%2B':
                storepart=part[3:]
        # store
        if name == '+save':
            # resolve special values
            if self.dparts['ip'] == u'%3Asource':
                self.dparts['ip'] = request.getClientIP()
            db = sqlite3.connect(serverdb)
            self.save(db.cursor())
            db.commit()
        return self

    def render_GET(self, request):
        return 'OK '+str(self.dparts)+self.messages

    def save(self, cur):
        try:
            cur.execute('INSERT INTO servers (name,service,ip,port) VALUES (?,?,?)', (self.dparts['server'], self.dparts['service'], self.dparts['ip'], self.dparts['port'])) 
            self.messages += '[saved]'
        except sqlite3.Error, e:
            self.messages += '[sqlite_error=' + e.args[0] + ']'


def all_services():
    db = sqlite3.connect(serverdb)
    db.row_factory = sqlite3.Row
    cur = db.cursor()
    cur.execute("SELECT COALESCE(si.ip, '0.0.0.0') as service_ip, COALESCE(si.port, '80') as service_port, srv.service as service,srv.name as name,srv.ip as ip, srv.port as port FROM servers srv LEFT OUTER JOIN services si ON si.name = srv.service ORDER BY srv.service")
    service = {'name':None}
    services = []
    servers = []
    rows = cur.fetchall()
    for row in rows:
        if service['name'] is None:
            service['name'] = row['service']
        if row['service'] != service['name']:
            service['servers'] = servers
            services.append(service)
            service = {}
            service['name'] = row['service']
            service['ip'] = row['service_ip']
            service['port'] = row['service_port']
            servers = []
        servers.append({'name':row['name'], 'ip':row['ip'],'port':row['port']})
    service = {}
    service['name'] = row['service']
    service['ip'] = row['service_ip']
    service['port'] = row['service_port']
    service['servers'] = servers
    services.append(service)
    return services
    

            

    
